#include "pexec.h"
#include <stdio.h>
#include <errno.h>

#define STDIN 0

int GetConnection ANSI_ARGS((PX_Ctx *, PX_Conn *, PX_Handler, int )), 
    UnlinkLocal ANSI_ARGS((PX_Ctx *, PX_Conn *, PX_Handler, int ));
extern int ReadCommand ANSI_ARGS(( PX_Ctx *, PX_Conn *, PX_Handler, int));
extern void *InitReadCommand( );

int main( argc, argv )
int  argc;
char **argv;
{
PX_Ctx     ctx;
PX_Handler exit_on_close, err_fd;
PX_Handler read_command;
PX_Handler unlinklocal, getconnection;
int        fd;
void       *sock_info;

PX_Ctx_init( &ctx );
PX_Monitor_setup( &ctx );

exit_on_close = PX_Handler_simple_create( PX_Hfcn_exit_on_close, (void *)0 );
err_fd        = PX_Handler_simple_create( PX_Hfcn_error_fd, (void *)0 );

/* Add a command interpreter to stdin */
read_command = PX_Handler_simple_create( ReadCommand, InitReadCommand(&ctx) );
PX_FD_add( STDIN,  PX_READ, read_command, err_fd, exit_on_close, 
	  (struct timeval *)0, &ctx );

/* Add a local socket */
fd = PX_FD_Create_local_socket( "/tmp/pexec", 1, &sock_info );
if (fd < 0) {
    fprintf( stderr, "Error creating local socket\n" );
    }
else {
    getconnection = PX_Handler_create( GetConnection, sock_info, (int (*)())0,
				   (void *)0, UnlinkLocal );
    unlinklocal   = PX_Handler_simple_create( UnlinkLocal, (void *)0 );
    PX_FD_add( fd, PX_READ, getconnection, unlinklocal, unlinklocal, 
	  (struct timeval *)0, &ctx );
    PX_Handler_free( unlinklocal );
    PX_Handler_free( getconnection );
    }

/* gselect is basically an infinite loop */
if (PX_Gselect( &ctx, (void *)0 )) {
    perror( "Gselect failed" );
    return 1;
    }
return 0;
}

/* This is called when a listen allows an accept */
int GetConnection( ctx, conn, handler, in_mask )
PX_Ctx     *ctx;
PX_Conn    *conn;
PX_Handler handler;
int        in_mask;
{
int fd;
PX_Handler err_fd, close_fd, fd_to_set;

fd = PX_FD_Accept_connection( conn->fd, handler->extra_data );
if (fd < 0) {
    fprintf( stderr, "accept errno = %d\n", errno );
    perror( "Accept failed" );
    return fd;
    }

close_fd      = PX_Handler_simple_create( PX_Hfcn_close_fd, (void *)0 );
fd_to_set     = PX_Handler_simple_create( PX_Hfcn_fd_to_set, (void *)0 );
err_fd        = PX_Handler_simple_create( PX_Hfcn_error_fd, (void *)0 );
PX_FD_add( fd, PX_READ, fd_to_set, err_fd, close_fd, (struct timeval *)0, 
	   ctx );
PX_FdAddToSet( fd_to_set, 1 );
PX_FdAddToSet( fd_to_set, 2 );
return 0;
}

int UnlinkLocal( ctx, conn, handler, err )
PX_Ctx     *ctx;
PX_Conn    *conn;
PX_Handler handler;
int        err;
{
unlink( "/tmp/pexec" );
return 0;
}
